@@ -29,7 +29,7 @@
 
 #' Individual Tree Detection Algorithm
 #'
-#' This function is made to be used in \link{tree_detection}. It implements an algorithm for tree
+#' This function is made to be used in \link{find_trees}. It implements an algorithm for tree
 #' detection based on a local maximum filter. The windows size can be fixed or variable and its
 #' shape can be square or circular. The internal algorithm works either with a raster or a point cloud.
 #' It is deeply inspired by Popescu & Wynne (2004) (see references).
@@ -61,14 +61,14 @@
 #' # =================
 #'
 #' # 5x5 m fixed window size
-#' ttops <- tree_detection(las, lmf(5))
+#' ttops <- find_trees(las, lmf(5))
 #'
 #' x <- plot(las)
 #' add_treetops3d(x, ttops)
 #'
 #' # variable windows size
 #' f <- function(x) { x * 0.07 + 3}
-#' ttops <- tree_detection(las, lmf(f))
+#' ttops <- find_trees(las, lmf(f))
 #'
 #' x <- plot(las)
 #' add_treetops3d(x, ttops)
@@ -81,14 +81,14 @@
 #' kernel <- matrix(1,3,3)
 #' chm <- raster::focal(chm, w = kernel, fun = median, na.rm = TRUE)
 #'
-#' ttops <- tree_detection(chm, lmf(5))
+#' ttops <- find_trees(chm, lmf(5))
 #'
 #' plot(chm, col = height.colors(30))
 #' plot(ttops, add = TRUE)
 #'
 #' # variable window size
 #' f <- function(x) { x * 0.07 + 3 }
-#' ttops <- tree_detection(chm, lmf(f))
+#' ttops <- find_trees(chm, lmf(f))
 #'
 #' plot(chm, col = height.colors(30))
 #' plot(ttops, add = TRUE)
@@ -123,7 +123,7 @@ lmf = function(ws, hmin = 2, shape = c("circular", "square"))
     return(C_lmf(las, ws, hmin, circ, getThread()))
   }
 
-  class(f) <- c("PointCloudBased", "IndividualTreeDetection", "OpenMP", "Algorithm", "lidR")
+  class(f) <- c(LIDRALGORITHMITD, LIDRALGORITHMOPENMP, LIDRALGORITHMPOINTCLOUDBASED)
   return(f)
 }
 
@@ -131,7 +131,7 @@ lmf = function(ws, hmin = 2, shape = c("circular", "square"))
 
 #' Individual Tree Detection Algorithm
 #'
-#' This function is made to be used in \link{tree_detection}. It implements an algorithm for manual
+#' This function is made to be used in \link{find_trees}. It implements an algorithm for manual
 #' tree detection. Users can pinpoint the tree top positions manually and interactively using the mouse.
 #' This is only suitable for small-sized plots. First the point cloud is displayed, then the user is
 #' invited to select a rectangular region of interest in the scene using the right mouse button.
@@ -153,11 +153,11 @@ lmf = function(ws, hmin = 2, shape = c("circular", "square"))
 #' las = readLAS(LASfile)
 #'
 #' # Full manual tree detection
-#' ttops = tree_detection(las, manual())
+#' ttops = find_trees(las, manual())
 #'
 #' # Automatic detection with manual correction
-#' ttops = tree_detection(las, lmf(5))
-#' ttops = tree_detection(las, manual(ttops))
+#' ttops = find_trees(las, lmf(5))
+#' ttops = find_trees(las, manual(ttops))
 #' }
 manual = function(detected = NULL, radius = 0.5, color = "red", ...) # nocov start
 {
@@ -249,6 +249,6 @@ manual = function(detected = NULL, radius = 0.5, color = "red", ...) # nocov sta
     return(output)
   }
 
-  class(f) <- c("function", "PointCloudBased", "IndividualTreeDetection", "Algorithm", "lidR")
+  class(f) <- c(LIDRALGORITHMITD, LIDRALGORITHMPOINTCLOUDBASED)
   return(f)
 } # nocov end
