@@ -182,6 +182,7 @@ grid_metrics.LAS = function(las, func, res = 20, start = c(0,0), filter = NULL)
     for (i in 1:nmetrics)
     {
       values <- vector(mode = class(metrics[[i]]), length = ncells)
+      values[] <- NA
       values[cells] <- metrics[[i]]
       output <- raster::setValues(output, values, layer = i)
     }
@@ -227,7 +228,8 @@ grid_metrics.LAScatalog = function(las, func, res = 20, start = c(0,0), filter =
     stop("The chunk size is too small. Process aborted.", call. = FALSE)
 
   # Enforce some options
-  opt_chunk_buffer(las) <- 0.1*alignment[["res"]]
+  if (opt_wall_to_wall(las))
+    opt_chunk_buffer(las) <- 0.1*alignment[["res"]]
 
   # Processing
   globals <- future::getGlobalsAndPackages(func)
