@@ -22,13 +22,16 @@
 #'
 #' @param las An object of class LAS
 #' @param func formula. An expression to be applied to each cell (see section "Parameter func").
-#' @param k integer. k-nearest neighbours
-#' @param r numeric. radius of the neighborhood sphere (not supported yet).
+#' @param k,r integer and numeric respectively for k-nearest neighbours and radius of the neighborhood
+#' sphere. If k is given and r is missing, compute the with the knn, if r is given and k is missing
+#' computes with a sphere neighborhood, if k and r are given computes with the knn and a limit on the
+#' search distance.
 #' @param xyz logical. Coordinates of each point are returned in addition to each metric. If
 #' \code{filter = NULL} coordinates are references to the original coordinates and do not occupy additional
 #' memory. If \code{filter != NULL} it obviously takes memory.
 #' @param filter formula of logical predicates. Enables the function to run only on points of interest
 #' in an optimized way. See also examples.
+#' @param ... unused.
 #'
 #' @section Parameter \code{func}:
 #' The function to be applied to each cell is a classical function (see examples) that
@@ -53,7 +56,7 @@
 #'
 #' # Computes the eigenvalues of the covariance matrix of the neighbouring
 #' # points and applies a test on these values. This function simulates the
-#' # 'shp_plane()' algorithm from 'lasdetectshape()'
+#' # 'shp_plane()' algorithm from 'segment_shape()'
 #' plane_metrics1 = function(x,y,z, th1 = 25, th2 = 6) {
 #'   xyz <- cbind(x,y,z)
 #'   cov_m <- cov(xyz)
@@ -67,7 +70,7 @@
 #' #> Computed in 6.3 seconds
 #'
 #' # We can verify that it returns the same as 'shp_plane'
-#' las <- lasdetectshape(las, shp_plane(k = 25), "planar")
+#' las <- segment_shape(las, shp_plane(k = 25), "planar")
 #' #> Computed in 0.1 second
 #'
 #' all.equal(M$planar, las$planar)
@@ -108,56 +111,86 @@
 #' dim(M1) # 13894 instead of 17182 previously.
 #'
 #' # is a memory-optimized equivalent to:
-#' first = lasfilterfirst(las)
+#' first = filter_first(las)
 #' M2 <- point_metrics(first, ~plane_metrics2(X,Y,Z), k = 25)
 #' all.equal(M1, M2)
 #' }
 #' @export
 #' @family metrics
-point_metrics <- function(las, func, k, r,  xyz = TRUE, filter = NULL) {
+point_metrics <- function(las, func, k, r,  xyz = FALSE, filter = NULL, ...) {
   UseMethod("point_metrics", las)
 }
 
 #' @export
-point_metrics.LAS <- function(las, func, k, r, xyz = TRUE, filter = NULL) {
-  # Defensive programming
-  if (!missing(k)) assert_is_a_number(k)
-  if (!missing(r)) assert_is_a_number(r)
-  if (!missing(k) && !missing(r))  stop("'k' and 'r' cannot be defined in the same time. It should be k or r.", call. = FALSE)
-  if (!missing(r)) stop("Radius search is not supported yet.", call. = FALSE)
+point_metrics.LAS <- function(las, func, k, r, xyz = FALSE, filter = NULL, ...) {
+
   if (missing(k) && missing(r))  stop("'k' or 'r' is missing", call. = FALSE)
 
+  # knn + radius
+  if (!missing(r) && !missing(k)) {
+    assert_is_a_number(k)
+    assert_is_a_number(r)
+    assert_all_are_positive(k)
+    assert_all_are_positive(r)
+    k <- as.integer(k)
+    mode <- 3L
+  }
+
+  # knn
+  if (!missing(k) && missing(r)) {
+    assert_is_a_number(k)
+    assert_all_are_positive(k)
+    k <- as.integer(k)
+    r <- 0
+    mode <- 0L
+  }
+
+  # radius
+  if (!missing(r) && missing(k)) {
+    assert_is_a_number(r)
+    assert_all_are_positive(r)
+    k <- 0
+    mode <- 1L
+  }
+
   assert_is_a_bool(xyz)
-  k <- as.integer(k)
-  stopifnot(k > 1)
+
   formula <- tryCatch(lazyeval::is_formula(func), error = function(e) FALSE)
   if (!formula) func <- lazyeval::f_capture(func)
 
   # Preparation of the objects
   func <- lazyeval::f_interp(func)
   call <- lazyeval::as_call(func)
-  data <- las@data
 
   # Memory allocation for the query. This memory will be recycled in each iteration
-  query <- data[1:k]
+  p <- list(...)
+  if (is.null(p$alloc))
+    n <- if (mode == 0L) k else as.integer(density(las) * pi * r^3)
+  else
+    n <- as.integer(p$alloc)
+
+  if (n < 1) n <- 1
 
   # Creation of a call environment
   env <- new.env(parent = parent.frame())
-  for (n in names(query)) assign(n, query[[n]], envir = env)
 
   filter <- parse_filter(las, filter)
-  output <- C_point_metrics(las, k, query, call, env, filter)
 
-  if (length(output[[1]]) == 1) {
+  output <- C_point_metrics(las, k, r, n, call, env, filter)
+
+  if (length(output[[1]]) == 1)
+  {
     name <- names(output[[1]])
     output <- data.table::data.table(unlist(output))
     if (!is.null(name)) data.table::setnames(output, name)
   }
-  else {
+  else
+  {
     output <- data.table::rbindlist(output)
   }
 
-  if (xyz) {
+  if (xyz)
+  {
     xyz <- coordinates3D(las)
     data.table::setDT(xyz)
     if (length(filter) > 1 && !all(filter)) xyz <- xyz[filter]
@@ -169,6 +202,19 @@ point_metrics.LAS <- function(las, func, k, r, xyz = TRUE, filter = NULL) {
     data.table::setcolorder(output, coln)
     output[]
   }
+  else
+  {
+    pointID <- NULL
+    colnames <- data.table::copy(names(output))
+
+    if (length(filter) > 1)
+      output[, pointID := which(filter)]
+    else
+      output[, pointID := 1:npoints(las)]
+
+    data.table::setcolorder(output, c("pointID", colnames))
+    output[]
+  }
 
   return(output)
 }
