@@ -9,7 +9,7 @@ test_that("grid_canopy with p2r() returns a georeferenced RasterLayer", {
   expect_equal(raster::res(x), c(4,4))
   expect_equal(dim(x), c(25,25,1))
   expect_equal(raster::extent(x), raster::extent(0,100,0,100))
-  expect_equal(x@crs, las@proj4string)
+  expect_equal(projection(x), projection(las))
   expect_equal(names(x), "Z")
 })
 
@@ -80,7 +80,7 @@ test_that("grid_canopy tin works both with LAS and LAScatalog", {
   expect_equal(raster::res(x), c(1,1))
   expect_equal(dim(x), c(88,88,1))
   expect_equal(raster::extent(x), raster::extent(481261,481349,3812922,3813010))
-  expect_equal(x@crs, las@proj4string)
+  expect_equal(projection(x), projection(las))
   expect_equal(names(x), "Z")
   expect_equal(x, y, tolerance = 0.00065)
 })
@@ -97,7 +97,7 @@ test_that("grid_canopy pit-free works both with LAS and LAScatalog", {
   expect_equal(raster::res(x), c(1,1))
   expect_equal(dim(x), c(88,88,1))
   expect_equal(raster::extent(x), raster::extent(481261,481349,3812922,3813010))
-  expect_equal(x@crs, las@proj4string)
+  expect_equal(projection(x), projection(las))
   expect_equal(names(x), "Z")
   expect_equal(x, y, tolerance = 0.00079)
 })
